package it.polito.bigdata.spark.sparkmllib;

import java.util.Arrays;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.ml.fpm.FPGrowth;
import org.apache.spark.ml.fpm.FPGrowthModel;

public class SparkDriver {

	@SuppressWarnings("resource")
	public static void main(String[] args) {
		String inputFile;
		String outputFolderItemsets;
		String outputFolderRules;
		double minSupport;
		double minConfidence;

		inputFile = args[0];
		outputFolderItemsets = args[1];
		outputFolderRules = args[2];
		minSupport = Double.parseDouble(args[3]);
		minConfidence = Double.parseDouble(args[4]);

		// Create a Spark Session object and set the name of the application
		// We use some Spark SQL transformation in this program
		SparkSession ss = SparkSession.builder().appName("MLlib - Itemset and Association rule mining").getOrCreate();

		// Create a Java Spark Context from the Spark Session
		// When a Spark Session has already been defined this method
		// is used to create the Java Spark Context
		JavaSparkContext sc = new JavaSparkContext(ss.sparkContext());

		// *************************
		// Itemset and rule mining
		// *************************

		JavaRDD<String> inputData = sc.textFile(inputFile);

		// Map each input line/data point of the input file to a Transaction
		// Transaction is characterized by the items "attribute" (data type:
		// List<String>)
		JavaRDD<Transaction> inputRDD = inputData.map(line -> {

			String[] items = line.split(" ");

			// Return a Transaction based on the content of the current line
			return new Transaction(Arrays.asList(items));
		});

		// Create a DataFrame based on the input data.
		Dataset<Row> transactionsData = ss.createDataFrame(inputRDD, Transaction.class).cache();

		// Create an FPGrowth object
		FPGrowth fp = new FPGrowth();

		// Set the value of min. support and min. confidence
		fp.setMinSupport(minSupport).setMinConfidence(minConfidence);

		// Extract frequent itemsets and association rules by invoking the fit
		// method of FPGrowth on the input data
		FPGrowthModel itemsetsAndRulesModel = fp.fit(transactionsData);

		// Retrieve the set of frequent itemsets
		Dataset<Row> frequentItemsets = itemsetsAndRulesModel.freqItemsets();

		// Retrieve the set of association rules
		Dataset<Row> frequentRules = itemsetsAndRulesModel.associationRules();

		// Save the itemset in an HDFS output folder
		JavaRDD<Row> itemsetsRDD = frequentItemsets.javaRDD();
		itemsetsRDD.saveAsTextFile(outputFolderItemsets);

		// Save the rules in an HDFS output folder
		JavaRDD<Row> rulesRDD = frequentRules.javaRDD();
		rulesRDD.saveAsTextFile(outputFolderRules);

		sc.close();

		ss.stop();
	}
}